@@ -1,14 +1,45 @@
 # coding: utf-8
 
-__all__ = ['BaseFetcherModel', 'CSSField', 'XPathField']
+__all__ = ['BaseFetcherModel', 'CSSField', 'XPathField', 'RedisCache']
 
 import json
+import logging
 import requests
+from collections import Sequence
 from redis import Redis
 from redis.exceptions import ConnectionError
 from scrapy.selector import Selector
 
-redis = Redis()
+logger = logging.getLogger(__name__)
+
+
+class NoCache(object):
+    def __init__(self, *args, **kwargs):
+        pass
+
+    def get(self, key):
+        return None
+
+    def set(self, key, value, expire=None):
+        pass
+
+
+class RedisCache(object):
+    def __init__(self, *args, **kwargs):
+        self.cache = Redis(*args, **kwargs)
+
+    def get(self, key):
+        try:
+            return self.cache.get(key)
+        except ConnectionError as e:
+            logger.error("Cant connect to Redis server %s", e)
+            return None
+
+    def set(self, key, value, expire=None):
+        try:
+            self.cache.set(key, value, expire)
+        except ConnectionError as e:
+            logger.error("Cant connect to Redis server %s", e)
 
 
 class Storage(dict):
@@ -33,9 +64,12 @@ class BaseField(object):
     Base for other selector fields
     """
 
-    def __init__(self, query, auto_extract=False):
+    def __init__(self, query,
+                 auto_extract=False, takes_first=False, processor=None):
         self.query = query
         self.auto_extract = auto_extract
+        self.takes_first = takes_first
+        self.processor = processor or (lambda untouched_data: untouched_data)
         self._data = self.selector = self._raw_data = None
 
     @property
@@ -44,9 +78,27 @@ class BaseField(object):
 
     def _parse(self, selector):
         parsed = self.parse(selector)
-        if self.auto_extract:
-            return parsed.extract()
-        return parsed
+        extracted = parsed.extract()
+        if self.takes_first and len(extracted) > 0:
+            for value in extracted:
+                if value is not None and value != '':
+                    return self._processor(value)
+        elif self.auto_extract:
+            return self._processor(extracted)
+        return self._processor(parsed)
+
+    def _processor(self, data):
+        """
+        runs the processor if defined
+        processor can be a list of functions to be chained
+        or a single function
+        """
+        if isinstance(self.processor, Sequence):
+            for function in self.processor:
+                data = function(data)
+        else:
+            data = self.processor(data)
+        return data
 
     def parse(self, selector):
         raise NotImplementedError("Must be implemented in child class")
@@ -104,7 +156,11 @@ class BaseFetcherModel(object):
 
     mappings = {}
 
-    def __init__(self, url=None, mappings=None, cache_fetch=False):
+    def __init__(self, url=None, mappings=None,
+                 cache_fetch=False,
+                 cache=NoCache,
+                 cache_args=None,
+                 cache_expire=None):
         self.load_fields()
         self.url = url
         self.refresh = False
@@ -112,6 +168,12 @@ class BaseFetcherModel(object):
         self._selector = None
         self.mappings = mappings or self.mappings.copy()
         self.cache_fetch = cache_fetch
+        self.cache_expire = cache_expire
+
+        if isinstance(cache, type):
+            self.cache = cache(**(cache_args or {}))
+        else:
+            self.cache = cache
 
     def load_fields(self):
         self._fields = []
@@ -122,15 +184,12 @@ class BaseFetcherModel(object):
 
     def fetch(self, url=None):
         url = self.url or url
-        try:
-            cached = redis.get(url)
-            if cached and self.cache_fetch:
-                return cached
-            response = requests.get(url)
-            if self.cache_fetch:
-                redis.set(url, response.content, ex=1800)
-        except ConnectionError:
-            response = requests.get(url)
+        cached = self.cache.get(url)
+        if cached and self.cache_fetch:
+            return cached
+        response = requests.get(url)
+        if self.cache_fetch:
+            self.cache.set(url, response.content, expire=self.cache_expire)
         return response.content
 
     @property
@@ -189,8 +248,15 @@ class BaseFetcherModel(object):
         for field_name, raw_selector in self._data.items():
             field_parser = getattr(self, 'parse_%s' % field_name, None)
             if field_parser:
-                parsed_data = field_parser(raw_selector)
-                self._data[field_name] = parsed_data
+                try:
+                    parsed_data = field_parser(raw_selector)
+                except Exception as e:
+                    logger.error(
+                        "Exception ocurred in parse_%s: %s", field_name, e
+                    )
+                    self._data[field_name] = raw_selector
+                else:
+                    self._data[field_name] = parsed_data
 
     def populate(self, obj, fields=None):
         fields = fields or self._data.keys()
